﻿using System;
using System.Collections.Generic;

using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace LD22 {

   class TexAtlas {

      public int FrameSizeX;
      public int FrameSizeY;
      public Vector2 FrameSizeV2;
      public Texture2D Texture;

      int cols;
      int rows;

      public int FrameCount { get { return cols * rows; } }

      public TexAtlas(Texture2D tex, int frameSizeX, int frameSizeY) {
         this.Texture = tex;
         this.FrameSizeX = frameSizeX;
         this.FrameSizeY = frameSizeY;
         this.FrameSizeV2 = new Vector2(frameSizeX, frameSizeY);
         this.cols = tex.Width / frameSizeX;
         this.rows = tex.Height / frameSizeY;
      }

      public Rectangle GetFrameRect(int index) {
         return new Rectangle(FrameSizeX * (index % cols), FrameSizeY * (index / cols), FrameSizeX, FrameSizeY);
      }
      public Rectangle GetFrameRect(int row, int col) {
         if (row < 0) return new Rectangle(FrameSizeX * (col % cols), FrameSizeY * (col / cols), FrameSizeX, FrameSizeY);
         return new Rectangle(FrameSizeX * col, FrameSizeY * row, FrameSizeX, FrameSizeY);
      }

   }

   class Animation {

      public TexAtlas Atlas;

      public string Name;
      public int Row;
      public int Frames;
      public float Duration;
      public bool Loop;

      public Animation(string name, TexAtlas atlas, int row, int frames, float duration, bool loop) {
         this.Name = name;
         this.Atlas = atlas;
         this.Row = row;
         this.Frames = frames;
         this.Duration = duration;
         this.Loop = loop;
      }

      public void Draw(SpriteBatch batch, Vector2 position, Color color, Vector2 origin, float scale, SpriteEffects flip, double time) {
         int frame = (int)(time / Duration * Frames);
         if (frame >= Frames) {
            if (Loop) frame = frame % Frames;
            else frame = Frames -1;
         }
         batch.Draw(Atlas.Texture, position, Atlas.GetFrameRect(Row, frame), color, 0f, Atlas.FrameSizeV2 * origin, scale, flip, 0f);
      }

   }

}
