﻿using System;
using System.Collections.Generic;

using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

namespace LD22 {

   class BackgroundLayer : SceneLayer {

      Texture2D bgtex;
      Texture2D moontex;

      public BackgroundLayer() {
         bgtex = Main.Game.Content.Load<Texture2D>("background/stars");
         moontex = Main.Game.Content.Load<Texture2D>("background/moon");
         GrabInput = false;
      }

      public override void Draw(SpriteBatch batch) {
         int width = Main.Game.ScreenWidth;
         int height = Main.Game.ScreenHeight;

         batch.Begin(SpriteSortMode.Immediate, BlendState.AlphaBlend, SamplerState.PointWrap, DepthStencilState.Default, RasterizerState.CullNone);
         batch.Draw(bgtex, Vector2.Zero, new Rectangle(0, 0, width, height), Color.White, 0, Vector2.Zero, 1.0f, SpriteEffects.None, 1f);
         batch.Draw(moontex, new Vector2(width - 100, 50), null, Color.White, 0, Vector2.Zero, 1.0f, SpriteEffects.None, 1f);
         batch.End();
      }

   }

   class ParallaxLayer : SceneLayer {

      Texture2D bgtex;
      float Speed;
      float Height;

      static SamplerState WrapX;

      public ParallaxLayer(Texture2D bgtex, float speed, float height) {
         this.bgtex = bgtex;
         this.Speed = speed;
         this.Height = height;
         GrabInput = false;
         if (WrapX == null) {
            WrapX = new SamplerState();
            WrapX.AddressU = TextureAddressMode.Wrap;
            WrapX.AddressV = TextureAddressMode.Clamp;
            WrapX.Filter = TextureFilter.Point;
         }
      }

      public override void Draw(SpriteBatch batch) {
         Vector2 offset = Main.Game.CameraPos * Speed;
         offset.Y -= (Main.Game.Level.Height - bgtex.Height) * (Speed) * (1f - this.Height);
         int width = Main.Game.ScreenWidth;
         int height = Main.Game.ScreenHeight;

         batch.Begin(SpriteSortMode.Immediate, BlendState.AlphaBlend, WrapX, DepthStencilState.Default, RasterizerState.CullNone);
         batch.Draw(bgtex, Vector2.Zero, new Rectangle((int)offset.X, (int)offset.Y, width, height), Color.White, 0, Vector2.Zero, 1.0f, SpriteEffects.None, 1f);
         batch.End();
      }

   }

}
