using System;
using System.Collections.Generic;
using System.Linq;

using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;

namespace LD22 {

   /// <summary>
   /// This is the main type for your game
   /// </summary>
   class GameLD22 : Microsoft.Xna.Framework.Game {

      GraphicsDeviceManager graphics;
 
      public double GameTime;
      public double LastElapsedTime;
      public float TimeMultiplier = 1f;

      public SpriteFont Font;

      public Level Level;
      public SceneManager Scene;

      public bool Pause = false;

      public int ScreenWidth;
      public int ScreenHeight;
      public bool SizeChanged = true;

      public Vector2 CameraPos;

      public int FPS = 0;
      int fpsCounter;
      float fpsTimer;

      public GameLD22() {
         graphics = new GraphicsDeviceManager(this);
         //graphics.PreferMultiSampling = true;
         graphics.GraphicsProfile = GraphicsProfile.Reach;
         graphics.PreferredBackBufferWidth = 1024;
         graphics.PreferredBackBufferHeight = 768;
         Window.Title = "LD22: Alone with... things";
         Window.AllowUserResizing = true;
         Window.ClientSizeChanged += Window_ClientSizeChanged;

         Content.RootDirectory = "Content";
         Scene = new SceneManager(this);
         Components.Add(Scene);

         Window_ClientSizeChanged(Window, EventArgs.Empty);
      }

      void Window_ClientSizeChanged(object sender, EventArgs e) {
         SizeChanged = true;
      }

      /// <summary>
      /// Allows the game to perform any initialization it needs to before starting to run.
      /// This is where it can query for any required services and load any non-graphic
      /// related content.  Calling base.Initialize will enumerate through any components
      /// and initialize them as well.
      /// </summary>
      protected override void Initialize() {
         // TODO: Add your initialization logic here
         base.Initialize();
      }

      /// <summary>
      /// LoadContent will be called once per game and is the place to load
      /// all of your content.
      /// </summary>
      protected override void LoadContent() {
         // Create a new SpriteBatch, which can be used to draw textures.
         //spriteBatch = new SpriteBatch(GraphicsDevice);
         Font = Content.Load<SpriteFont>("fonts/SpriteFont1");
         ContentCache.LoadContent(Content);
         Level = new Level();
         Level.Load("level1");

         Scene.Layers.Add(new BackgroundLayer());
         Scene.Layers.Add(new ParallaxLayer(Content.Load<Texture2D>("background/skyline_black"), 0.1f, 0.4f));
         Scene.Layers.Add(new ParallaxLayer(Content.Load<Texture2D>("background/skyline_gray"), 0.4f, 0.25f));
         Scene.Layers.Add(new LevelSceneLayer());
         Scene.Layers.Add(new HUDLayer());
         Scene.Layers.Add(new PlayAgainLayer("You wake up alone. Thankfully you've got the ability to bring objects to life.", "Arrow keys for movement, animate furniture with D (or B on a GamePad)", "Start Game", "Exit", Color.White, 0f));
      }

      /// <summary>
      /// UnloadContent will be called once per game and is the place to unload
      /// all content.
      /// </summary>
      protected override void UnloadContent() {
         //spriteBatch.Dispose();
         ContentCache.UnloadContent();
      }

      /// <summary>
      /// Allows the game to run logic such as updating the world,
      /// checking for collisions, gathering input, and playing audio.
      /// </summary>
      /// <param name="gameTime">Provides a snapshot of timing values.</param>
      protected override void Update(GameTime gameTime) {
         if (Pause) {
            this.LastElapsedTime = 0f;
         } else {
            this.LastElapsedTime = gameTime.ElapsedGameTime.TotalSeconds * TimeMultiplier;
            this.GameTime += LastElapsedTime;
         }

         

         InputHandler.Update();

         // Allows the game to exit
         if (InputHandler.GamePad1.Buttons.Back == ButtonState.Pressed) this.Exit();


         base.Update(gameTime);
      }

      /// <summary>
      /// This is called when the game should draw itself.
      /// </summary>
      /// <param name="gameTime">Provides a snapshot of timing values.</param>
      protected override void Draw(GameTime gameTime) {
         fpsCounter++;
         fpsTimer += (float)gameTime.ElapsedGameTime.TotalSeconds;
         if (fpsTimer >= 1f) {
            FPS = fpsCounter;
            fpsTimer -= 1f;
            fpsCounter = 0;
         }
         if (SizeChanged) {
            SizeChanged = false;
            ScreenWidth = graphics.GraphicsDevice.PresentationParameters.BackBufferWidth;
            ScreenHeight = graphics.GraphicsDevice.PresentationParameters.BackBufferHeight;
         }
         //GraphicsDevice.Clear(Color.CornflowerBlue);

         // TODO: Add your drawing code here

         base.Draw(gameTime);
      }

   }
}
