﻿using System;
using System.Collections.Generic;

using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace LD22 {

   class Particle {

      public Texture2D Texture;
      public Color Color = Color.White;
      public Vector2 Position;
      public Vector2 Velocity;
      public float Weight = 0.2f;
      public float Scale = 1f;
      public float Rotation;
      public float RotationSpeed = 0f;
      public float Lifetime = 5f;
      public float Grow = 0f;

      public Particle(Texture2D texture, Vector2 position) {
         this.Texture = texture;
         this.Position = position;
         this.Rotation = Functions.rnd(MathHelper.Pi);
      }

      public void Update(Level level, float elapsedTime) {
         Lifetime -= elapsedTime;
         if (Lifetime <= 0f || Scale <= 0f) level.Particles.Remove(this);
         else if (Lifetime < 1f) Color.A = (byte)(Lifetime * 255f);
         Velocity += level.Gravity * Weight * elapsedTime;
         Position += Velocity * elapsedTime;
         Rotation += RotationSpeed * elapsedTime;
         Rotation += Grow * elapsedTime;
      }

      public void Draw(SpriteBatch batch) {
         batch.Draw(Texture, Position - Main.Game.CameraPos, null, Color, Rotation, 0.5f * new Vector2(Texture.Width, Texture.Height), Scale, SpriteEffects.None, 0f);
      }

   }

}
