﻿using System;
using System.Collections.Generic;

using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;

namespace LD22 {

   static class ContentCache {

      public static Texture2D Particle_Magic;

      public static Texture2D UI_Heart;
      public static Texture2D UI_HeartEmpty;

      public static ObjectTemplate Boy;
      public static ObjectTemplate Kitty;
      public static ObjectTemplate Chair;
      public static ObjectTemplate Table;
      public static ObjectTemplate Dog;
      public static ObjectTemplate Heart;
      public static ObjectTemplate AlarmClock;

      public static SoundEffect Sound_BoyJump;
      public static SoundEffect Sound_Land;
      public static SoundEffect Sound_Run;
      public static SoundEffect Sound_Magic;
      public static SoundEffect Sound_PickupHealth;
      public static SoundEffectInstance SoundLoop_Run; 

      public static void LoadContent(ContentManager Content) {

         Particle_Magic = Content.Load<Texture2D>("particles/magic");

         UI_Heart = Content.Load<Texture2D>("ui/heart");
         UI_HeartEmpty = Content.Load<Texture2D>("ui/heartempty");

         Sound_BoyJump = Content.Load<SoundEffect>("sounds/jump");
         Sound_Land = Content.Load<SoundEffect>("sounds/impact");
         Sound_Run = Content.Load<SoundEffect>("sounds/run");
         Sound_Magic = Content.Load<SoundEffect>("sounds/magic");
         Sound_PickupHealth = Content.Load<SoundEffect>("sounds/pickuphealth");

         var boy_atlas = new TexAtlas(Content.Load<Texture2D>("chars/boy"), 32, 32);
         Boy = new ObjectTemplate();
         Boy.AddAnimation(new Animation("idle", boy_atlas, 0, 4, 8f, true));
         Boy.AddAnimation(new Animation("walk", boy_atlas, 1, 4, 0.5f, true));
         Boy.AddAnimation(new Animation("fall", boy_atlas, 2, 1, 1f, true));
         Boy.AddAnimation(new Animation("jump", boy_atlas, 3, 1, 1f, true));
         Boy.AddAnimation(new Animation("damage", boy_atlas, 4, 1, 1f, true));
         Boy.AddAnimation(new Animation("die", boy_atlas, 5, 4, 1f, false));
         Boy.JumpSound = Sound_BoyJump;
         Boy.LandSound = Sound_Land;
         Boy.WalkingSound = Sound_Run;

         var kitty_atlas = new TexAtlas(Content.Load<Texture2D>("chars/kitty"), 32, 32);
         Kitty = new ObjectTemplate();
         Kitty.AddAnimation(new Animation("idle", kitty_atlas, 0, 1, 1f, true));
         Kitty.DefaultAI = (level, me) => {
            if ((level.Player.Position - me.Position).Length() < 16) {
               level.PlayerMaxHealth++;
               level.PlayerHealth++;
               Sound_Magic.Play();
               level.SpawnParticles(ContentCache.Particle_Magic, me.Position, 30, 40f, 2f, (p) => {
                  p.Scale = Functions.fuzz(0.8f, 0.2f);
                  p.RotationSpeed = Functions.rnd(6f);
                  p.Color = Color.CornflowerBlue; //new Color(236, 0, 140);
               });
               level.Objects.Remove(me);
            }
         };


         var heart_atlas = new TexAtlas(Content.Load<Texture2D>("items/heartanim"), 16, 16);
         Heart = new ObjectTemplate();
         Heart.AddAnimation(new Animation("idle", heart_atlas, -1, 12, 2f, true));
         Heart.CoollisionRect = new Rectangle(-8, -8, 16, 16);
         Heart.Origin = new Vector2(0.5f, 0.5f);
         Heart.Scale = 1f;
         Heart.DefaultAI = (level, me) => {
            if (level.PlayerHealth >= level.PlayerMaxHealth) return;
            if ((level.Player.Position - me.Position).Length() < 16) {
               level.PlayerHealth++;
               Sound_PickupHealth.Play();
               level.Objects.Remove(me);
            }
         };

         var clock_atlas = new TexAtlas(Content.Load<Texture2D>("items/alarmclock"), 16, 16);
         AlarmClock = new ObjectTemplate();
         AlarmClock.AddAnimation(new Animation("idle", clock_atlas, 0, 1, 2f, true));
         AlarmClock.CoollisionRect = new Rectangle(-16, -16, 32, 32);
         AlarmClock.Origin = new Vector2(0.5f, 0.5f);
         AlarmClock.DefaultAI = (level, me) => {
            if ((level.Player.Position - me.Position).Length() < 16) {
               Sound_Magic.Play();
               level.SpawnParticles(ContentCache.Particle_Magic, me.Position, 30, 40f, 2f, (p) => {
                  p.Scale = Functions.fuzz(0.8f, 0.2f);
                  p.RotationSpeed = Functions.rnd(6f);
                  p.Color = Color.CornflowerBlue; //new Color(236, 0, 140);
               });
               level.Objects.Remove(me);
               Main.Game.Scene.Layers.Add(new PlayAgainLayer("The best nightmares are those that end when you wake up.", "You have won! Play again?", "Yes", "No", Color.White, 0f));
            }
         };

         var chair_atlas = new TexAtlas(Content.Load<Texture2D>("chars/chair"), 32, 32);
         Chair = new ObjectTemplate();
         Chair.AddAnimation(new Animation("inactive", chair_atlas, 0, 1, 1f, true));
         Chair.AddAnimation(new Animation("idle", chair_atlas, 1, 1, 8f, true));
         Chair.AddAnimation(new Animation("walk", chair_atlas, 2, 4, 0.25f, true));
         Chair.AddAnimation(new Animation("fall", chair_atlas, 3, 1, 1f, true));
         Chair.AddAnimation(new Animation("jump", chair_atlas, 3, 1, 1f, true));
         Chair.LandSound = Sound_Land;
         Chair.WalkingSound = Sound_Run;
         Chair.Companion = true;
         Chair.Acceleration = 128f;
         Chair.MaxSpeed = 10f; // 128f;
         Chair.CoollisionRect = new Rectangle(-16, -40, 32, 52);
         Chair.Origin = new Vector2(0.5f, 0.75f);
         Chair.DefaultAnimation = "inactive";
         Chair.CompanionActivation = (level, me) => {
            me.PlayAnimation("idle");
            level.Player = me;
            level.Objects.Remove(level.PlayerChar);
         };

         var table_atlas = new TexAtlas(Content.Load<Texture2D>("chars/table"), 64, 64);
         Table = new ObjectTemplate();
         Table.AddAnimation(new Animation("idle", table_atlas, 0, 1, 1f, true));
         Table.AddAnimation(new Animation("walk", table_atlas, 1, 4, 0.5f, true));
         Table.LandSound = Sound_Land;
         Table.WalkingSound = Sound_Run;
         Table.Companion = true;
         Table.Acceleration = 50f;
         Table.MaxSpeed = 4f;
         Table.CoollisionRect = new Rectangle(-37, -2, 74, 30);
         Table.FlatSurface = true;
         Table.Origin = new Vector2(0.5f, 0.75f);
         Table.CompanionActivation = (level2, me2) => {
            me2.AI = (level, me) => {
               var ppos = level.Player.Position;
               if ((System.Math.Abs(ppos.X - me.Position.X)) > 96) {
                  if (ppos.X < me.Position.X) me.MoveLeft();
                  else if (ppos.X > me.Position.X) me.MoveRight();
               }
               if (System.Math.Abs(me.Velocity.X) < 0.1f) me.PlayAnimation("idle");
            };
         };

         var dog_atlas = new TexAtlas(Content.Load<Texture2D>("chars/dog"), 32, 32);
         Dog = new ObjectTemplate();
         Dog.AddAnimation(new Animation("idle", dog_atlas, 0, 4, 8f, true));
         Dog.AddAnimation(new Animation("walk", dog_atlas, 1, 4, 0.5f, true));
         Dog.LandSound = Sound_Land;
         Dog.AttackSound = Content.Load<SoundEffect>("sounds/dogbite");
         Dog.ChargeSound = Content.Load<SoundEffect>("sounds/dogbark");
         Dog.Acceleration = 70f;
         Dog.MaxSpeed = 6f; // 70f;
         Dog.JumpStrength = 8f;
         Dog.CoollisionRect = new Rectangle(-16, -20, 32, 32);
         Dog.Origin = new Vector2(0.5f, 0.75f);
         Dog.DefaultAI = (level, me) => {
            var ppos = level.Player.Position;
            if ((ppos - me.Position).Length() < 320) {
               if (me.IsIdle && me.Template.ChargeSound != null) me.Template.ChargeSound.Play(0.5f, Functions.rnd(0.2f), 0f);
               if (ppos.X < me.Position.X) me.MoveLeft();
               else if (ppos.X > me.Position.X) me.MoveRight();
               if (Functions.Random.NextDouble() <= Main.Game.LastElapsedTime) me.Jump();
               if (me.GetRect().Intersects(level.Player.GetRect())) {
                  if (level.Player.Damage(me.LookingRight) && me.Template.AttackSound != null) me.Template.AttackSound.Play(0.75f, Functions.rnd(0.2f), 0f);
               }
            }
            if (me.Velocity.X == 0f) me.PlayAnimation("idle");
         };

         SoundLoop_Run = Sound_Run.CreateInstance();
         SoundLoop_Run.Volume = 0.5f;
         SoundLoop_Run.IsLooped = true;
      }
      public static void UnloadContent() {

      }

   }

}
