﻿using System;
using System.Collections.Generic;

using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;

namespace LD22 {

   class Level {

      public GameLD22 Game { get { return Main.Game; } }

      public ObjectInstance PlayerChar;
      public ObjectInstance Player;
      public ObjectInstance Kitty;
      public ObjectInstance Companion;
      public ObjectInstance LastSurfaceCompanion;

      public int TilesX;
      public int TilesY;
      public int Width;
      public int Height;

      public int PlayerHealth = CONST.START_HEALTH;
      public int PlayerMaxHealth = CONST.MAX_HEALTH;

      public byte[][] Data;
      AdjaMask[][] Adjacency;
      Material[] Materials;

      public Vector2 Gravity = new Vector2(0f, 48f);

      SpriteBatch fgBatch;
      SpriteBatch bgBatch;

#if DEBUG
      public Texture2D DummyTileBG;
#endif

      public List<ObjectInstance> Objects = new List<ObjectInstance>();
      public List<Particle> Particles = new List<Particle>();

      public ObjectInstance CreateObject(ObjectTemplate template, Vector2 position) {
         var obj = new ObjectInstance(template);
         obj.Position = position;
         obj.PlayAnimation(template.DefaultAnimation);
         obj.AI = template.DefaultAI;
         Objects.Add(obj);
         return obj;
      }
      public ObjectInstance CreateObject(ObjectTemplate template, int x, int y) {
         var pos = new Vector2(x * CONST.TILE_SIZE, y * CONST.TILE_SIZE); ;
         return CreateObject(template, pos);
      }


      public void Load(string name) {
         Objects.Clear();

         PlayerHealth = CONST.START_HEALTH;
         PlayerMaxHealth = CONST.MAX_HEALTH;
         PlayerChar = CreateObject(ContentCache.Boy, 118, 189);
         Player = PlayerChar;

         CreateObject(ContentCache.Chair, 50, 189);
         CreateObject(ContentCache.Table, 53, 189);

         CreateObject(ContentCache.Heart, 58, 179);
         CreateObject(ContentCache.Heart, 124, 179);

         CreateObject(ContentCache.Chair, 118, 189);
         CreateObject(ContentCache.Table, 121, 189);
         CreateObject(ContentCache.Chair, 124, 189).LookingRight = false;

         CreateObject(ContentCache.Dog, 183, 236);
         CreateObject(ContentCache.Dog, 78, 187);
         Kitty = CreateObject(ContentCache.Kitty, 95, 229);

         CreateObject(ContentCache.Chair, 180, 187);
         CreateObject(ContentCache.Table, 193, 177);
         CreateObject(ContentCache.Chair, 195, 177).LookingRight = false;

         CreateObject(ContentCache.Chair, 291, 199).LookingRight = false;
         CreateObject(ContentCache.Table, 289, 199);
         CreateObject(ContentCache.Chair, 288, 204);

         CreateObject(ContentCache.Dog, 320, 238);
         CreateObject(ContentCache.Dog, 347, 237);

         CreateObject(ContentCache.Dog, 390, 230);

         CreateObject(ContentCache.Table, 342, 179);
         CreateObject(ContentCache.Chair, 344, 184);

         CreateObject(ContentCache.Heart, 347, 185);

         CreateObject(ContentCache.Heart, 441, 193);

         CreateObject(ContentCache.Heart, 306, 234);

         CreateObject(ContentCache.AlarmClock, 288, 251);

         var content = Game.Content;

#if DEBUG
         DummyTileBG = content.Load<Texture2D>("tiles/dummy_bg");
#endif

         fgBatch = new SpriteBatch(Game.GraphicsDevice);
         bgBatch = new SpriteBatch(Game.GraphicsDevice);

         Materials = new Material[256];
         Materials[128].Load("tiles/bricks");
         Materials[129].Load("tiles/earth");
         Materials[130].Load("tiles/glass");
         Materials[131].Load("tiles/leaves");
         Materials[132].Load("tiles/trunk");

         using (var fs = new System.IO.FileStream("Content/levels/" + name + ".bmp",  System.IO.FileMode.Open, System.IO.FileAccess.Read, System.IO.FileShare.Read)) {
            using (var br = new System.IO.BinaryReader(fs)) {

               if (br.ReadChar() != 'B' || br.ReadChar() != 'M') throw new Exception("Invalid level file! Must be a 256-indexed bitmap!");
               fs.Position += 8;
               int address = br.ReadInt32();
               br.ReadInt32();
               TilesX = br.ReadInt32();
               TilesY = br.ReadInt32();
               br.ReadInt16(); // always 1
               if (br.ReadInt16() != 8) throw new Exception("The level bitmap must be 256 indexed colors!");
               int padding = TilesX % 4;

               Data = new byte[TilesY][];
               Adjacency = new AdjaMask[TilesY][];
               fs.Position = address;
               for (int i = TilesY-1; i >= 0; i--) {
                  Data[i] = br.ReadBytes(TilesX);
                  if (padding > 0) br.ReadBytes(padding);
                  Adjacency[i] = new AdjaMask[TilesX];
               }
               for (int y = 0; y < TilesY; y++) {
                  for (int x = 0; x < TilesX; x++) {
                     Adjacency[y][x] = CalcAdjacency(x, y);
                  }
               }

               Width = TilesX * CONST.TILE_SIZE;
               Height = TilesY * CONST.TILE_SIZE;
            }
            
         }

      }

      public void DamagePlayer() {
         if (PlayerHealth <= 0) return;
         PlayerHealth--;
         if (PlayerHealth <= 0) {
            DeactivateCompanion();
            Player.LockAnimationTime = 0f;
            if (Player.PlayAnimation("die")) Player.LockAnimationTime = 9999;
            Game.Scene.Layers.Add(new PlayAgainLayer("You died!", "Play again?", "Yes", "No", Color.Red, 3f));
         }
      }

      public bool CollisionY(float x, float y, int width, bool downward) {
         int ty = (int)(y / CONST.TILE_SIZE);
         if (ty < 0 || ty >= TilesY) return true;
         int tx = (int)(x / CONST.TILE_SIZE);
         //if (tx < 0 || tx >= TilesX) return true; // don't need to check tx and txe bounds, this is only for vertical movement
         int txe = (int)((x + width - 1) / CONST.TILE_SIZE);
         for (; tx <= txe; tx++) {
            if (Data[ty][tx] >= CONST.FIRST_SOLID_MATID) return true;
         }
         if (downward && LastSurfaceCompanion != null) {
            var srect = LastSurfaceCompanion.GetRect();
            if ((int)y < srect.Y || (int)y > srect.Y + 1) return false;
            if (x >= srect.Right || (x + width) <= srect.X) return false;
            return true;
         }
         return false;
      }
      public bool CollisionX(float x, float y, int height) {
         int tx = (int)(x / CONST.TILE_SIZE);
         if (tx < 0 || tx >= TilesX) return true;
         int ty = (int)(y / CONST.TILE_SIZE);
         //if (ty < 0 || ty >= TilesY) return true; // don't need to check ty and tye bounds, this is only for horizontal movement
         int tye = (int)((y + height - 1) / CONST.TILE_SIZE);
         for (; ty <= tye; ty++) {
            if (Data[ty][tx] >= CONST.FIRST_SOLID_MATID) return true;
         }
         return false;
      }

      AdjaMask CalcAdjacency(int x, int y) {
         //int min = Data[y][x];
         //if (min == 0) return AdjaMask.None;
         //min = (min >= 128 ? 128 : 1);
         AdjaMask res = AdjaMask.None;

         if (x <= 0 || Data[y][x - 1] >= CONST.FIRST_SOLID_MATID) res |= AdjaMask.Left | AdjaMask.BGLeft;
         else if (Data[y][x - 1] >= CONST.FIRST_BG_MATID) res |= AdjaMask.BGLeft;

         if (x >= (TilesX - 1) || Data[y][x + 1] >= CONST.FIRST_SOLID_MATID) res |= AdjaMask.Right | AdjaMask.BGRight;
         else if (Data[y][x + 1] >= CONST.FIRST_BG_MATID) res |= AdjaMask.BGRight;

         if (y <= 0 || Data[y - 1][x] >= CONST.FIRST_SOLID_MATID) res |= AdjaMask.Top | AdjaMask.BGTop;
         else if (Data[y - 1][x] >= CONST.FIRST_BG_MATID) res |= AdjaMask.BGTop;

         if (y >= (TilesY - 1) || Data[y + 1][x] >= CONST.FIRST_SOLID_MATID) res |= AdjaMask.Bottom | AdjaMask.BGBottom;
         else if (Data[y + 1][x] >= CONST.FIRST_BG_MATID) res |= AdjaMask.BGBottom;

         return res;
      }

      public void Draw(SpriteBatch batch) {
         if (PlayerChar == null) return;

         var camPos = Player.Position - new Vector2(Game.ScreenWidth / 2, Game.ScreenHeight / 2);
         if (camPos.X < 0f) camPos.X = 0f;
         else if (camPos.X > (Width - Game.ScreenWidth)) camPos.X = Width - Game.ScreenWidth;
         if (camPos.Y < 0f) camPos.Y = 0f;
         else if (camPos.Y > (Height - Game.ScreenHeight)) camPos.Y = Height - Game.ScreenHeight;
         Game.CameraPos = camPos;

         int minX = (int)(camPos.X / CONST.TILE_SIZE);
         int minY = (int)(camPos.Y / CONST.TILE_SIZE);
         int maxX = (int)((camPos.X + Game.ScreenWidth - 1) / CONST.TILE_SIZE);
         int maxY = (int)((camPos.Y + Game.ScreenHeight - 1) / CONST.TILE_SIZE);

         fgBatch.Begin(SpriteSortMode.Texture, BlendState.AlphaBlend, SamplerState.PointClamp, DepthStencilState.Default, RasterizerState.CullNone);
         bgBatch.Begin(SpriteSortMode.Texture, BlendState.AlphaBlend, SamplerState.PointClamp, DepthStencilState.Default, RasterizerState.CullNone);

         for (int iY = minY; iY <= maxY; iY++) {
            for (int iX = minX; iX <= maxX; iX++) {
               DrawTile(iX, iY, Data[iY][iX], ref camPos);
            }
         }
         bgBatch.End();
         

         batch.Begin(SpriteSortMode.Deferred, BlendState.AlphaBlend, SamplerState.PointClamp, DepthStencilState.Default, RasterizerState.CullNone);
         for (int i = 0; i < Objects.Count; i++) {
            Objects[i].Draw(batch);
         }
         batch.End();
         batch.Begin(SpriteSortMode.Deferred, BlendState.AlphaBlend, SamplerState.PointClamp, DepthStencilState.Default, RasterizerState.CullNone);
         Player.Draw(batch);
         batch.End();

         batch.Begin(SpriteSortMode.Texture, BlendState.Additive, SamplerState.LinearClamp, DepthStencilState.Default, RasterizerState.CullNone);
         //batch.Begin();
         for (int i = 0; i < Particles.Count; i++) {
            Particles[i].Draw(batch);
         }
         batch.End();

         fgBatch.End();
      }

      public void SpawnParticles(Texture2D tex, Vector2 pos, int count, float speed, float lifetime, Action<Particle> apply = null) {
         for (int i = 0; i < count; i++) {
            var dir = Functions.rndV2(4f);
            var p = new Particle(tex, pos + dir);
            p.Velocity = dir * 0.25f * speed;
            p.Lifetime = lifetime * 0.25f + (float)Functions.Random.NextDouble() * lifetime * 0.75f;
            if (apply != null) apply(p);
            Particles.Add(p);
         }
      }

      public void DrawTile(int x, int y, int matid) {
         var camPos = Game.CameraPos;
         DrawTile(x, y, matid, ref camPos);
      }
      void DrawTile(int x, int y, int matid, ref Vector2 camPos) {
         if (matid >= 128) {
            AdjaMask adj = Adjacency[y][x];
            Rectangle? src = Materials[matid].Atlas.GetFrameRect((int)(adj & AdjaMask.All));
            fgBatch.Draw(Materials[matid].Atlas.Texture, new Rectangle(x * CONST.TILE_SIZE - (int)camPos.X, y * CONST.TILE_SIZE - (int)camPos.Y, CONST.TILE_SIZE, CONST.TILE_SIZE), src, Color.White);
            if ((adj & AdjaMask.All) != AdjaMask.All && (AdjaMask)((int)adj >> 4) != (adj & AdjaMask.All)) {
               src = Materials[matid].Atlas.GetFrameRect((int)adj >> 4);
               bgBatch.Draw(Materials[matid].Atlas.Texture, new Rectangle(x * CONST.TILE_SIZE - (int)camPos.X, y * CONST.TILE_SIZE - (int)camPos.Y, CONST.TILE_SIZE, CONST.TILE_SIZE), src, Color.Gray);
            }
         } else if (matid > 0) {
            matid += 127;
            AdjaMask adj = Adjacency[y][x];
            Rectangle? src = Materials[matid].Atlas.GetFrameRect((int)adj >> 4);
            bgBatch.Draw(Materials[matid].Atlas.Texture, new Rectangle(x * CONST.TILE_SIZE - (int)camPos.X, y * CONST.TILE_SIZE - (int)camPos.Y, CONST.TILE_SIZE, CONST.TILE_SIZE), src, Color.Gray);
         }
      }

      public ObjectInstance GetObjectAt(Rectangle rect, Func<ObjectInstance, bool> condition = null) {
         bool intersect;
         for (int i = 0; i < Objects.Count; i++) {
            Objects[i].GetRect().Intersects(ref rect, out intersect);
            if (intersect && (condition == null || condition(Objects[i]))) return Objects[i];
         }
         return null;
      }

      public void DeactivateCompanion() {
         if (Companion == null) return;
         if (Player == Companion) {
            Player = PlayerChar;
            Objects.Add(PlayerChar);
            PlayerChar.Position = Companion.Position;
            PlayerChar.LookingRight = Companion.LookingRight;
            PlayerChar.Velocity = Companion.Velocity;
         }
         if (!Companion.PlayAnimation("inactive")) Companion.PlayAnimation("idle");
         Companion.AI = null;
         Companion = null;
      }
      public void ActivateCompanion() {
         var obj = GetObjectAt(Player.GetRect(), (o) => (o != Player));
         if (obj == null || !obj.Template.Companion) return;
         ActivateCompanion(obj);
      }
      public void ActivateCompanion(ObjectInstance newCompanion) {
         if (newCompanion == null) return;
         this.Companion = newCompanion;
         if (Companion.Template.CompanionActivation != null) Companion.Template.CompanionActivation(this, Companion);
         SpawnParticles(ContentCache.Particle_Magic, Companion.Position, 30, 40f, 2f, (p) => {
            p.Scale = Functions.fuzz(0.8f, 0.2f);
            p.RotationSpeed = Functions.rnd(6f);
            p.Color = Color.Violet; //new Color(236, 0, 140);
         });
         ContentCache.Sound_Magic.Play(1f, Functions.rnd(0.2f), 0f);
         if (Companion.Template.FlatSurface) LastSurfaceCompanion = Companion;
      }

      public void Update(float elapsedTime, bool handleInput) {
         if (PlayerChar == null) return;
            bool wasOnGround = Player.IsOnGround;

            if (handleInput && PlayerHealth > 0) {
               bool running = false;
               if (InputHandler.Keyboard.IsKeyDown(Keys.Left) || InputHandler.GamePad1.IsButtonDown(Buttons.DPadLeft)) {
                  //Player.Velocity -= new Vector2(Player.Template.MovementSpeed * elapsedTime, 0f);
                  //if (Player.IsOnGround) Player.PlayAnimation("walk");
                  //Player.LookingRight = false;
                  Player.MoveLeft();
                  running = true;

               } else if (InputHandler.Keyboard.IsKeyDown(Keys.Right) || InputHandler.GamePad1.IsButtonDown(Buttons.DPadRight)) {
                  //Player.Velocity += new Vector2(Player.Template.MovementSpeed * elapsedTime, 0f);
                  //if (Player.IsOnGround) Player.PlayAnimation("walk");
                  //Player.LookingRight = true;
                  Player.MoveRight();
                  running = true;

               } else {
                  if (Player.IsOnGround) Player.PlayAnimation("idle");
               }

               if (running && Player.IsOnGround) {
                  if (ContentCache.SoundLoop_Run.State != SoundState.Playing) ContentCache.SoundLoop_Run.Play();
               } else if (ContentCache.SoundLoop_Run.State == SoundState.Playing) ContentCache.SoundLoop_Run.Stop();

               if (Player.IsOnGround) {
                  if (InputHandler.KeyPressed(Keys.Up) || InputHandler.KeyPressed(Keys.S) || InputHandler.ButtonPressed(Buttons.A)) Player.Jump(); // InputHandler.ButtonPressed(Buttons.DPadUp) ||
               }

               if (InputHandler.KeyPressed(Keys.D) || InputHandler.ButtonPressed(Buttons.B)) {
                  if (Companion != null) DeactivateCompanion();
                  else ActivateCompanion();
               }
            }

            if (!Player.IsOnGround) {
               if (Player.Velocity.Y < 0f) Player.PlayAnimation("jump");
               else Player.PlayAnimation("fall");
            }

         //if (InputHandler.Keyboard.IsKeyDown(Keys.W) || InputHandler.GamePad1.DPad.Up == ButtonState.Pressed) {
         //   Player.Velocity -= new Vector2(0f, 128f * elapsedTime);
         //   //Player.PlayAnimation("jump");
         //   //} else if (InputHandler.Keyboard.IsKeyDown(Keys.S) || InputHandler.GamePad1.DPad.Down == ButtonState.Pressed) {
         //   //   Player.Velocity += new Vector2(0f, 64f * elapsedTime);
         //   //   Player.PlayAnimation("jump");
         //}

            for (int i = Objects.Count - 1; i >= 0; i--) {
               Objects[i].Update(this, elapsedTime);
            }
            if (!wasOnGround && Player.IsOnGround) ContentCache.Sound_Land.Play(0.4f, Functions.rnd(0.6f), 0f);

            for (int i = Particles.Count - 1; i >= 0; i--) {
               Particles[i].Update(this, elapsedTime);
            }
        
      }

   }

}
