﻿using System;
using System.Collections.Generic;

using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace LD22 {

   class ObjectInstance {

      public ObjectTemplate Template;
      Animation Animation;
      double AnimStart;
      Action AnimEndCallback;

      public Vector2 Position;
      public bool LookingRight = true;
      public Vector2 Velocity;

      public Color Color = Color.White;
      

      public bool IsOnGround;
      //public int Health = 3;
      public float Invulnerability;
      public float LockAnimationTime;

      public Action<Level, ObjectInstance> AI;

      public bool IsIdle { get { return Animation.Name == "idle"; } }

      public ObjectInstance(ObjectTemplate animations) {
         this.Template = animations;
         this.Animation = Template.GetAnimation("idle");
      }

      public void Update(Level level, float elapsedTime) {
         if (Invulnerability > 0f) Invulnerability -= elapsedTime;
         if (LockAnimationTime > 0f) LockAnimationTime -= elapsedTime;
         if (AI != null) AI(level, this);

         if (IsOnGround) Velocity.X *= 1f - elapsedTime*10f;
         Velocity += level.Gravity * Template.Weight * elapsedTime;
         var srcRect = GetRect();
         var trgRect = GetRect(Velocity);

         if (trgRect.Y < srcRect.Y) {
            for (int iY = srcRect.Y; iY >= trgRect.Y; iY--) {
               if (level.CollisionY(srcRect.X, iY, Template.CoollisionRect.Width, false)) {
                  Velocity.Y = 0f;
                  goto endy;
               }
            }
         } else if (trgRect.Y > srcRect.Y) {
            int trg = trgRect.Bottom - 1;
            for (int iY = srcRect.Bottom - 1; iY <= trg; iY++) {
               if (level.CollisionY(srcRect.X, iY, Template.CoollisionRect.Width, true)) {
                  Velocity.Y = 0f;
                  goto endy;
               }
            }
         }
         Position.Y += Velocity.Y;
endy:

         int posY = (int)(Position.Y) + Template.CoollisionRect.Y;
         if (trgRect.X < srcRect.X) {
            for (int iX = srcRect.X; iX >= trgRect.X; iX--) {
               if (level.CollisionX(iX, posY, Template.CoollisionRect.Height)) {
                  Velocity.X = 0f;
                  goto endx;
               }
            }
         } else if (trgRect.X > srcRect.X) {
            int trg = trgRect.Right - 1;
            for (int iX = srcRect.Right - 1; iX <= trg; iX++) {
               if (level.CollisionX(iX, posY, Template.CoollisionRect.Height)) {
                  Velocity.X = 0f;
                  goto endx;
               }
            }
         }
         Position.X += Velocity.X;
endx: ;
         IsOnGround = level.CollisionY((int)(Position.X) + Template.CoollisionRect.X, posY + Template.CoollisionRect.Height, Template.CoollisionRect.Width, true);
      }


      public bool PlayAnimation(Animation anim, Action callback = null) {
         if (anim == null || LockAnimationTime > 0f) return false; // anim = Template.GetAnimation("idle");
         if (anim == Animation) return false;
         this.AnimStart = Main.Game.GameTime;
         this.Animation = anim;
         this.AnimEndCallback = callback;
         return true;
      }
      public bool PlayAnimation(string anim, Action callback = null) {
         if (this.Animation != null && this.Animation.Name == anim) return false;
         return PlayAnimation(Template.GetAnimation(anim), callback);
      }

      public void Jump() {
         if (!IsOnGround) return;
         Velocity.Y -= Template.JumpStrength;
         PlayAnimation("jump");
         if (Template.JumpSound != null) Template.JumpSound.Play(0.75f, Functions.rnd(0.1f), 0f);
      }

      public void MoveLeft() {
         if (Velocity.X > -Template.MaxSpeed) Velocity.X -= Template.Acceleration * (float)Main.Game.LastElapsedTime;
         if (IsOnGround) PlayAnimation("walk");
         LookingRight = false;
      }
      public void MoveRight() {
         if (Velocity.X < Template.MaxSpeed) Velocity.X += Template.Acceleration * (float)Main.Game.LastElapsedTime;
         if (IsOnGround) PlayAnimation("walk");
         LookingRight = true;
      }

      public bool Damage(bool pushRight) {
         if (Invulnerability > 0f) return false;
         Invulnerability += 2f;
         if (PlayAnimation("damage")) LockAnimationTime = 0.5f;
         if (this == Main.Game.Level.Player) Main.Game.Level.DamagePlayer();
         if (Velocity.Y > -16f) Velocity.Y = -16f;
         if (pushRight) Velocity.X += 8f;
         else Velocity.X -= 8f;
         return true;
      }

      public Rectangle GetRect() {
         var rect = Template.CoollisionRect;
         rect.X += (int)Position.X;
         rect.Y += (int)Position.Y;
         return rect;
      }
      public Rectangle GetRect(Vector2 offset) {
         var rect = Template.CoollisionRect;
         rect.X += (int)(Position.X + offset.X); //(int)Position.X + (int)offset.X;
         rect.Y += (int)(Position.Y + offset.Y); //(int)Position.Y + (int)offset.Y;
         return rect;
      }

      public void Draw(SpriteBatch batch) {
         if (Animation == null) return;
         Animation.Draw(batch, Position - Main.Game.CameraPos, Color, Template.Origin, Template.Scale, (LookingRight ? SpriteEffects.None : SpriteEffects.FlipHorizontally), Main.Game.GameTime - AnimStart);

#if DEBUG
         if (InputHandler.Keyboard.IsKeyDown(Microsoft.Xna.Framework.Input.Keys.LeftAlt)) {
            batch.Draw(Main.Game.Level.DummyTileBG, GetRect(-Main.Game.CameraPos), Color.Red);
         }
#endif

      }

   }

}
